'use strict';
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.expectFunctionToThrow = exports.expectErrorViaFunction = exports.expectThrowViaFunction = exports.expectErrorViaCallback = exports.expectViaCallback = exports.throwErrorViaCallback = exports.returnErrorViaCallback = exports.errorViaCallback = exports.completeViaCallback = exports.returnViaCallback = exports.errorEqual = exports.contains = exports.deepEqual = exports.nullish = exports.undef = exports.lt = exports.gt = exports.lte = exports.gte = exports.equal = exports.logComparison = exports.log = exports.inspect = exports.color = exports.useColors = exports.bool = exports.isTTY = exports.isNode = exports.wait = void 0;
// Polyfill for older node versions
if (Array.prototype.includes == null) {
    // eslint-disable-next-line no-extend-native
    Array.prototype.includes = function (searchElement) {
        return this.indexOf(searchElement) !== -1;
    };
}
// Import
var assert_1 = require("assert");
var util_1 = require("util");
var ansi = __importStar(require("@bevry/ansi"));
var errlop_1 = __importDefault(require("errlop"));
var process_1 = require("process");
/** Alias for setTimeout with paramaters reversed. */
function wait(delay, fn) {
    // @ts-ignore
    return setTimeout(fn, delay);
}
exports.wait = wait;
/** Whether or not we are running in the node environment */
function isNode() {
    return Boolean(process_1.versions && process_1.versions.node);
}
exports.isNode = isNode;
/** Whether or not stdout and stderr are interactive. */
function isTTY() {
    return (isNode() &&
        process_1.stdout &&
        process_1.stdout.isTTY === true &&
        process_1.stderr &&
        process_1.stderr.isTTY === true);
}
exports.isTTY = isTTY;
/** Convert a value to its boolean equivalent */
function bool(value) {
    if (value === 'no' || value === 'false' || value === 'n' || value === 'N') {
        return false;
    }
    if (value == null || value === '' || value === 'null' || value === 'NULL')
        return null;
    // node.js 15 compatibility
    // which sets COLOR to 0 (is TTY) 1 (not TTY)
    // which is the opposite of what one would reasonable expect
    // so ignore such pollution, such that decision is determined by another factor
    if (value === '0')
        return null;
    if (value === '1')
        return null;
    // return boolean version
    return Boolean(value);
}
exports.bool = bool;
/** Whether or not colors are desired on this environment */
function useColors() {
    var _a, _b;
    // if unsupported, return false
    if (!isNode())
        return false;
    // if disabled, return false
    if (process_1.argv.includes('--no-colors') || process_1.argv.includes('--no-color'))
        return false;
    // if unspecified, use default (tty)
    return (_b = (_a = bool(process_1.env.COLOR)) !== null && _a !== void 0 ? _a : bool(process_1.env.COLORS)) !== null && _b !== void 0 ? _b : isTTY();
}
exports.useColors = useColors;
/** Applies the color to the value if desired */
function color(value, color) {
    return useColors() ? color(value) : value;
}
exports.color = color;
/**
 * Checks to see if a value is an object
 * https://github.com/bevry/typechecker/blob/69008d42927749d7e21cfe9816e478dd8d15ab88/source/index.js#L22-L30
 */
function isObject(value) {
    // null is object, hence the extra check
    return value !== null && typeof value === 'object';
}
/**
 * Return a stringified version of the value with indentation and colors where applicable.
 * Colors will be applied if the environment supports it (--no-colors not present and TTY).
 * For the available options, refer to https://nodejs.org/dist/latest-v14.x/docs/api/util.html#util_util_inspect_object_options for Node.js
 */
function inspect(value, opts) {
    if (opts === void 0) { opts = {}; }
    // If the terminal supports colours, and the user hasn't specified, then default to a sensible default
    var colors = useColors();
    var depth = 50;
    // Inspect and return using our defaults
    return (0, util_1.inspect)(value, __assign({ colors: colors, depth: depth }, opts));
}
exports.inspect = inspect;
/** Log the inspected values of each of the arguments to stdout */
function log() {
    var args = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        args[_i] = arguments[_i];
    }
    if (isNode() && process_1.env.ASSERT_SILENCE)
        return;
    for (var i = 0; i < args.length; ++i) {
        console.log(inspect(args[i]));
    }
}
exports.log = log;
/** Output a comparison of the failed result to stderr */
function logComparison(actual, expected, error) {
    if (isNode() && process_1.env.ASSERT_SILENCE)
        return;
    var lines = [
        '------------------------------------',
        'Comparison Error:',
        color(error.stack || error.message || error, ansi.green),
        '',
    ];
    lines.push('Comparison Actual:', inspect(actual), '', 'Comparison Expected:', inspect(expected), '------------------------------------');
    // Work for node
    if (isNode() && process_1.stderr) {
        process_1.stderr.write(lines.join('\n') + '\n');
    }
    // Work for browsers
    else {
        console.log(lines.join('\n'));
    }
}
exports.logComparison = logComparison;
/** Same as assert.strictEqual in that it performs a strict equals check, but if a failure occurs it will output detailed information */
function equal(actual, expected, testName, next) {
    if (testName === void 0) { testName = 'equal assertion'; }
    try {
        (0, assert_1.strictEqual)(actual, expected, testName);
    }
    catch (checkError) {
        logComparison(actual, expected, checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.equal = equal;
/** Is greater than or equal to */
function gte(actual, expected, testName, next) {
    if (testName === void 0) { testName = 'is greater than or equal to assertion'; }
    try {
        (0, assert_1.strictEqual)(actual >= expected, true, testName);
    }
    catch (checkError) {
        logComparison(actual, expected, checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.gte = gte;
/** Is less than or equal to */
function lte(actual, expected, testName, next) {
    if (testName === void 0) { testName = 'is less than or equal to assertion'; }
    try {
        (0, assert_1.strictEqual)(actual <= expected, true, testName);
    }
    catch (checkError) {
        logComparison(actual, expected, checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.lte = lte;
/** Is greater than */
function gt(actual, expected, testName, next) {
    if (testName === void 0) { testName = 'is greater than assertion'; }
    try {
        (0, assert_1.strictEqual)(actual > expected, true, testName);
    }
    catch (checkError) {
        logComparison(actual, expected, checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.gt = gt;
/** Is less than */
function lt(actual, expected, testName, next) {
    if (testName === void 0) { testName = 'is less than assertion'; }
    try {
        (0, assert_1.strictEqual)(actual < expected, true, testName);
    }
    catch (checkError) {
        logComparison(actual, expected, checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.lt = lt;
/** Ensure what is passed is undefined, otherwise fail and output what it is */
function undef(actual, testName, next) {
    if (testName === void 0) { testName = 'undef assertion'; }
    try {
        (0, assert_1.strictEqual)(typeof actual, 'undefined', testName);
    }
    catch (checkError) {
        logComparison(actual, 'undefined', checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.undef = undef;
/** Ensure what is passed is undefined or null, otherwise fail and output what it is */
function nullish(actual, testName, next) {
    if (testName === void 0) { testName = 'nullish assertion'; }
    try {
        (0, assert_1.strictEqual)(typeof actual, 'undefined', testName);
    }
    catch (e1) {
        try {
            (0, assert_1.strictEqual)(actual, null, testName);
        }
        catch (e2) {
            var error = new errlop_1.default(e2, e1);
            logComparison(actual, 'nullish', error);
            if (next) {
                next(error);
                return;
            }
            else {
                throw error;
            }
        }
    }
    if (next)
        next();
}
exports.nullish = nullish;
/** Same as assert.deepStrictEqual in that it performs a deep strict equals check, but if a failure occurs it will output detailed information */
function deepEqual(actual, expected, testName, next) {
    if (testName === void 0) { testName = 'deep equal assertion'; }
    try {
        (0, assert_1.deepStrictEqual)(actual, expected, testName);
    }
    catch (checkError) {
        logComparison(actual, expected, checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.deepEqual = deepEqual;
/** Checks to see if the actual result contains the expected result .*/
function contains(actual, expected, testName, next) {
    if (testName === void 0) { testName = 'contains assertion'; }
    if (testName == null)
        testName = "Expected [".concat(actual, "] to contain [").concat(expected, "]");
    try {
        (0, assert_1.ok)(actual.indexOf(expected) !== -1, testName);
    }
    catch (checkError) {
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.contains = contains;
/** Checks to see if an error was as expected, if a failure occurs it will output detailed information */
function errorEqual(actualError, expectedError, testName, next) {
    if (testName === void 0) { testName = 'error equal assertion'; }
    var expectedErrorMessage, actualErrorMessage;
    if (expectedError) {
        if (expectedError instanceof Error) {
            expectedErrorMessage = expectedError.message;
        }
        else {
            expectedErrorMessage = expectedError;
            expectedError = new Error(expectedErrorMessage);
        }
    }
    if (actualError) {
        if (actualError instanceof Error) {
            actualErrorMessage = actualError.message;
        }
        else {
            actualErrorMessage = actualError;
            actualError = new Error(actualErrorMessage);
        }
    }
    try {
        if (actualErrorMessage && expectedErrorMessage) {
            contains(actualErrorMessage, expectedErrorMessage, testName);
        }
        else {
            equal(actualError, expectedError || null, testName);
        }
    }
    catch (checkError) {
        logComparison(actualError && (actualError.stack || actualError.message || actualError), expectedErrorMessage, checkError);
        if (next) {
            next(checkError);
            return;
        }
        else {
            throw checkError;
        }
    }
    if (next)
        next();
}
exports.errorEqual = errorEqual;
/** Generate a callback that will return the specified value. */
function returnViaCallback(value) {
    return function () {
        return value;
    };
}
exports.returnViaCallback = returnViaCallback;
/** Generate a callback that will receive a completion callback whcih it will call with the specified result after the specified delay. */
/* eslint no-magic-numbers:0 */
function completeViaCallback(value, delay) {
    if (delay === void 0) { delay = 100; }
    return function (complete) {
        wait(delay, function () {
            complete(null, value);
        });
    };
}
exports.completeViaCallback = completeViaCallback;
/** Generate a callback that will receive a completion callback which it will call with the passed error after the specified delay. */
/* eslint no-magic-numbers:0 */
function errorViaCallback(error, delay) {
    if (delay === void 0) { delay = 100; }
    return function (complete) {
        wait(delay, function () {
            complete(error);
        });
    };
}
exports.errorViaCallback = errorViaCallback;
/** Generate a callback that return an error instance with the specified message/error. */
function returnErrorViaCallback(error) {
    if (error === void 0) { error = 'an error occurred'; }
    return function () {
        if (error instanceof Error) {
            return error;
        }
        else {
            return new Error(error);
        }
    };
}
exports.returnErrorViaCallback = returnErrorViaCallback;
/** Generate a callback that throw an error instance with the specified message/error. */
function throwErrorViaCallback(error) {
    if (error === void 0) { error = 'an error occurred'; }
    return function () {
        if (error instanceof Error) {
            throw error;
        }
        else {
            throw new Error(error);
        }
    };
}
exports.throwErrorViaCallback = throwErrorViaCallback;
/** Generate a callback that will check the arguments it received with the arguments specified, if a failure occurs it will output detailed information. */
function expectViaCallback() {
    var expected = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        expected[_i] = arguments[_i];
    }
    return function () {
        var actual = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            actual[_i] = arguments[_i];
        }
        return deepEqual(actual, expected);
    };
}
exports.expectViaCallback = expectViaCallback;
/**
 * Generate a callback that will check its error (the actual error) against the passed error (the expected error).
 * If a failure occurs it will output detailed information. */
function expectErrorViaCallback(expected, testName, next) {
    if (testName === void 0) { testName = 'expect error via callback assertion'; }
    return function (actual) {
        return errorEqual(actual, expected, testName, next);
    };
}
exports.expectErrorViaCallback = expectErrorViaCallback;
/** Expect the passed function to throw an error at some point. */
function expectThrowViaFunction(expected, fn, testName, next) {
    if (testName === void 0) { testName = 'expect error via function assertion'; }
    var actual = null;
    try {
        fn();
    }
    catch (error) {
        actual = error;
    }
    errorEqual(actual, expected, testName, next);
}
exports.expectThrowViaFunction = expectThrowViaFunction;
/** @deprecated Use {@link expectErrorViaFunction} instead */
function expectErrorViaFunction() {
    throw new Error('expectErrorViaFunction has been deprecated, use expectThrowViaFunction instead');
}
exports.expectErrorViaFunction = expectErrorViaFunction;
/** @deprecated Use {@link expectErrorViaFunction} instead */
function expectFunctionToThrow() {
    throw new Error('expectFunctionToThrow has been deprecated, use expectThrowViaFunction instead');
}
exports.expectFunctionToThrow = expectFunctionToThrow;
